import html
from time import sleep

import requests
from lxml import html as lxmhtml

from saml2art.utils.httputils import validate_response


class DuoMFA:
    def __init__(self, duo_host):
        self.duo_host = duo_host
        pass

    def authenticate(self, referral_url, duoSignature):
        duo_auth_url = "https://%s/frame/web/v1/auth" % self.duo_host
        duo_signatures = duoSignature.split(":")

        initialize_data = {"parent": referral_url,
                           "java_version": "",
                           "flash_version": "",
                           "screen_resolution_width": "3008",
                           "screen_resolution_height": "1692",
                           "color_depth": "24",
                           "tx": duo_signatures[0]}
        duo_auth_response = validate_response(requests.post(duo_auth_url, data=initialize_data))

        duo_sid = self.extract_sid(duo_auth_response)

        duo_push_url = "https://%s/frame/prompt" % self.duo_host
        duo_push_data = {"sid": duo_sid,
                         "device": "phone1",
                         "factor": "Duo Push",
                         "out_of_date": "false"}

        duo_auth_response = validate_response(requests.post(duo_push_url, data=duo_push_data))
        duoTxStat = duo_auth_response.json()['stat']

        duo_tx_id = duo_auth_response.json()['response']['txid']
        duo_status_url = "https://%s/frame/status" % self.duo_host
        duo_status_data = {
            "sid": duo_sid,
            "txid": duo_tx_id}
        duo_status_response = requests.post(duo_status_url, data=duo_status_data)

        duoTxResult = duo_status_response.json()['response'].get('result', None)
        duoResultURL = duo_status_response.json()['response'].get('result_url', None)
        print(duo_status_response.json()['response']['status'])

        if duoTxResult != "SUCCESS":
            while True:
                sleep(3)

                duo_status_response = requests.post(duo_status_url, data=duo_status_data)

                duoTxResult = duo_status_response.json()['response'].get('result', None)
                duoResultURL = duo_status_response.json()['response'].get('result_url', None)
                print(duo_status_response.json()['response']['status'])

                if duoTxResult == "FAILURE":
                    # TODO Throw!
                    pass

                if duoTxResult == "SUCCESS":
                    break

        duoResultURL = ("https://%s%s" % (self.duo_host, duoResultURL))
        duoResultResponse = requests.post(duoResultURL, data=duo_status_data)

        duo_tx_cookie = duoResultResponse.json()['response']['cookie']
        return "%s:%s" % (duo_tx_cookie, duo_signatures[1])

    def extract_sid(self, duo_auth_response):
        tree = lxmhtml.fromstring(duo_auth_response.content)
        duo_sid = html.unescape(tree.xpath('//input[@name="sid"]/@value')[0])
        return duo_sid



import netrc
import os


class NetrcExporter:
    def export(self, path, host, username, password):
        print("Updating [%s]..." % path, end="")
        netrc_file_path = os.path.expanduser(path)
        if os.path.exists(netrc_file_path):
            netrc_file = netrc.netrc(netrc_file_path)
        else:
            netrc_file = netrc.netrc()
        netrc_file.hosts[host] = (username, None, password)
        with open(os.path.expanduser(path), "w+") as file:
            file.write(netrc_file.__repr__())
        print("Done!")



from argparse_prompt import PromptParser

from saml2art.commands.login import login_args, login_saml2art
from saml2art.commands.configure import configure_saml2art, configure_args
from saml2art.config.config import SAML2ArtConfig

version = "0.0.2"
DEFAULT_CONFIG_FILE_PATH = "~/.saml2art"


def main():
    print("Artifactory Token Generator %s" % version)
    parser = PromptParser(
        description='A command line tool to help with generating API Key to Artifactory with SAML access.')
    subparsers = parser.add_subparsers(help="", dest="action")
    subparsers.required = True
    login_args(subparsers)
    configure_args(subparsers)
    args = parser.parse_args()
    if args.action == "login":
        config = SAML2ArtConfig()
        config.load(DEFAULT_CONFIG_FILE_PATH)
        login_saml2art(config, args)
    elif args.action == "configure":
        configure_saml2art(args, DEFAULT_CONFIG_FILE_PATH)
    else:
        parser.print_help()




import html
from time import sleep

import requests
from lxml import html as lxmhtml

from saml2art.mfa.duomfa import DuoMFA
from saml2art.utils.httputils import validate_response


class OktaIdP:

    def __init__(self, okta_org_host):
        self.okta_org_host = okta_org_host
        self.okta_session_token = ""

    def authenticate(self, user, pwd):
        auth_response = validate_response(requests.post('https://%s/api/v1/authn' % self.okta_org_host,
                                                        json={"username": user, "password": pwd}))
        auth_status = auth_response.json()['status']
        if auth_status == 'SUCCESS':
            self.okta_session_token = auth_response.json()['sessionToken']
            print("Authenticated!")
        elif auth_status == 'MFA_REQUIRED':
            factor_id = self.handle_mfa(auth_response)

            verify_data = {"stateToken": auth_response.json()['stateToken']}
            okta_verify_url = 'https://%s/api/v1/authn/factors/%s/verify' % (self.okta_org_host, factor_id)
            verify_response = validate_response(requests.post(okta_verify_url, json=verify_data))

            self.okta_session_token = verify_response.json()['sessionToken']
        else:
            raise Exception("Error: Got [%s] auth status" % auth_status)

    def handle_mfa(self, auth_response):
        factor_id = auth_response.json()["_embedded"]['factors'][0]['id']
        verify_response = validate_response(
            requests.post('https://%s/api/v1/authn/factors/%s/verify' % (self.okta_org_host, factor_id),
                          json={"stateToken": auth_response.json()['stateToken']}))
        duo_host = verify_response.json()['_embedded']['factor']['_embedded']['verification']['host']
        duo_signature = verify_response.json()['_embedded']['factor']['_embedded']['verification']['signature']
        duo_callback_url = \
            verify_response.json()['_embedded']['factor']['_embedded']['verification']['_links']['complete']['href']
        duo_mfa = DuoMFA(duo_host)
        duo_signature_response = duo_mfa.authenticate("https://%s/signin/verify/duo/web" % self.okta_org_host,
                                                      duo_signature)
        duo_response_to_okta_data = {
            "id": factor_id,
            "stateToken": auth_response.json()['stateToken'],
            "sig_response": duo_signature_response}
        validate_response(requests.post(duo_callback_url, data=duo_response_to_okta_data))
        return factor_id

    def get_saml_assertion(self, redirectUrl):
        okta_session_redirect_url = "https://%s/login/sessionCookieRedirect" % self.okta_org_host

        redirect_response = validate_response(
            requests.get(okta_session_redirect_url, params={"checkAccountSetupComplete": "true",
                                                            "token": self.okta_session_token,
                                                            "redirectUrl": redirectUrl}))

        return lxmhtml.fromstring(redirect_response.content).xpath('//input[@name="SAMLResponse"]/@value')[0]
